@@ -16,7 +16,7 @@
  */
 
 /*
- * $Id: RegularExpression.cpp 695949 2008-09-16 15:57:44Z borisk $
+ * $Id$
  */
 
 // ---------------------------------------------------------------------------
@@ -140,7 +140,7 @@ RegularExpression::Context::Context(Context* src) :
     }
     if(src->fMatch)
     {
-        fMatch=new Match(*src->fMatch);
+        fMatch=new (fMemoryManager) Match(*src->fMatch);
         fAdoptMatch=true;
     }
 }
@@ -152,30 +152,46 @@ RegularExpression::Context& RegularExpression::Context::operator=(const RegularE
         fStart=other.fStart;
         fLimit=other.fLimit;
         fLength=other.fLength;
-        fSize=other.fSize;
         fStringMaxLen=other.fStringMaxLen;
         fString=other.fString;
         fOptions=other.fOptions;
-        if (fOffsets)
-            fMemoryManager->deallocate(fOffsets);//delete [] fOffsets;
-        fOffsets=0;
-        if (fAdoptMatch)
-            delete fMatch;
-        fMatch=0;
-        fAdoptMatch=false;
 
-        fMemoryManager=other.fMemoryManager;
-        if(other.fOffsets)
+        // if offset and match are already allocated with the right size, reuse them 
+        // (fMatch can be provided by the user to get the data back)
+        if(fMatch && other.fMatch && fMatch->getNoGroups()==other.fMatch->getNoGroups())
+            *fMatch=*other.fMatch;
+        else
+        {
+            if (fAdoptMatch)
+                delete fMatch;
+            fMatch=0;
+            if(other.fMatch)
+            {
+                fMatch=new (other.fMemoryManager) Match(*other.fMatch);
+                fAdoptMatch=true;
+            }
+        }
+
+        if (fOffsets && other.fOffsets && fSize==other.fSize)
         {
-            fOffsets = (int*) fMemoryManager->allocate(fSize* sizeof(int));
             for (int i = 0; i< fSize; i++)
                 fOffsets[i] = other.fOffsets[i];
         }
-        if(other.fMatch)
+        else
         {
-            fMatch=new Match(*other.fMatch);
-            fAdoptMatch=true;
+            if(fOffsets)
+                fMemoryManager->deallocate(fOffsets);//delete [] fOffsets;
+            fOffsets=0;
+            fSize=other.fSize;
+            if(other.fOffsets)
+            {
+                fOffsets = (int*) other.fMemoryManager->allocate(fSize* sizeof(int));
+                for (int i = 0; i< fSize; i++)
+                    fOffsets[i] = other.fOffsets[i];
+            }
         }
+
+        fMemoryManager=other.fMemoryManager;
     }
 
     return *this;
@@ -552,7 +568,7 @@ bool RegularExpression::matches(const XMLCh* const expression, const XMLSize_t s
 
             if (context.fMatch != 0) {
                 context.fMatch->setStartPos(0, ret);
-                context.fMatch->setEndPos(0, (int)(ret + strLength));
+                context.fMatch->setEndPos(0, (int)(ret + XMLString::stringLen(fPattern)));
             }
             return true;
         }
@@ -1430,14 +1446,32 @@ Op* RegularExpression::compile(const Token* const token, Op* const next,
 
     switch(tokenType) {
     case Token::T_DOT:
+        ret = fOpFactory.createDotOp();
+        ret->setNextOp(next);
+        break;
     case Token::T_CHAR:
+        ret = fOpFactory.createCharOp(token->getChar());
+        ret->setNextOp(next);
+        break;
     case Token::T_ANCHOR:
+        ret = fOpFactory.createAnchorOp(token->getChar());
+        ret->setNextOp(next);
+        break;
     case Token::T_RANGE:
     case Token::T_NRANGE:
+        ret = fOpFactory.createRangeOp(token);
+        ret->setNextOp(next);
+        break;
     case Token::T_STRING:
+        ret = fOpFactory.createStringOp(token->getString());
+        ret->setNextOp(next);
+        break;
     case Token::T_BACKREFERENCE:
+        ret = fOpFactory.createBackReferenceOp(token->getReferenceNo());
+        ret->setNextOp(next);
+        break;
     case Token::T_EMPTY:
-        ret = compileSingle(token, next, tokenType);
+        ret = next;
         break;
     case Token::T_CONCAT:
         ret = compileConcat(token, next, reverse);
@@ -1563,7 +1597,7 @@ bool RegularExpression::doTokenOverlap(const Op* op, Token* token)
             {
                 try
                 {
-                    RangeToken tempRange(Token::T_RANGE, fMemoryManager);
+                    RangeToken tempRange(t1->getTokenType(), fMemoryManager);
                     tempRange.mergeRanges(t1);
                     tempRange.intersectRanges((RangeToken*)token);
                     return !tempRange.empty();
